# -*- coding: utf-8 -*-
import datetime

from django.contrib.syndication.feeds import Feed
from django.http import Http404

from reiare.asamashie.models import AsamashiWord, AsamashiRakutenXML, AsamashiRakutenGenreXML


class LatestAsamashiWords(Feed):
    title = 'あさましいあれ（仮題）'
    link = '/asamashie/'
    description = '【楽天ウェブサービス】RAKUTEN WEBSERVICEを用いて、“気になるキーワード”の検索結果から日々淡々と楽天市場の商品を紹介し続けてみるページです。検索結果は一日数回程度任意のタイミングで更新されます。商品の説明内容、及び実際の販売は各リンク先の楽天市場さんのお店によるものですので一切関知できません。が、結構新しい情報だと思いますよ。'
    author_name = 'reiare.net/asamashie'
    title_template = 'feeds/asamashie_latest_title.html'
    description_template = None

    def items(self):
        return AsamashiWord.objects.all()[:10]

    def item_pubdate(self, item):
        return item.modified

class LatestItemsByKeyword(LatestAsamashiWords):
    title_template = 'feeds/asamashie_keyword_title.html'
    description_template = 'feeds/asamashie_keyword_description.html'
    
    pubdate = None
    
    def get_object(self, bits):
        if len(bits) != 1:
            raise ObjectDoesNotExist
        asamashi_word = AsamashiWord.objects.get(pk=bits[0])
        self.title = asamashi_word.keyword + " - " + self.title
        self.link = asamashi_word.get_absolute_url() + 'utd/'
        asamashi_rakuten_xml, created \
            = AsamashiRakutenXML.objects.get_or_create(asamashi_word=asamashi_word,
                                                       sort='-updateTimestamp',
                                                       min_price__isnull=True, max_price__isnull=True,
                                                       genre_id__isnull=True, page_count=1)
        self.pubdate = asamashi_rakuten_xml.modified
        return asamashi_rakuten_xml
        

    def items(self, obj):
        try:
            return obj.get_dict()['items'][:10]
        except:
            return []

    def item_link(self, item):
        return item['affiliate_url']

    def item_pubdate(self):
        #return self.pubdate
        return None

class LatestItemsByGenre(LatestItemsByKeyword):
    def get_object(self, bits):
        if len(bits) != 1 and len(bits) != 2:
            raise ObjectDoesNotExist

        asamashi_rakuten_genre_xml = AsamashiRakutenGenreXML.objects.get(genre_id=bits[0])
        self.title = asamashi_rakuten_genre_xml.get_dict()['current_genre_name'] + ' - ' + self.title
        self.link += 'gid' + asamashi_rakuten_genre_xml.get_dict()['current_genre_id'] + '/'
        if len(bits) == 1:
            asamashi_rakuten_xml, created \
                = AsamashiRakutenXML.objects.get_or_create(genre_id=bits[0], asamashi_word__isnull=True,
                                                           sort='-updateTimestamp', page_count=1,
                                                           min_price__isnull=True, max_price__isnull=True)
        elif len(bits) == 2:
            asamashi_word = AsamashiWord.objects.get(pk=bits[1])
            self.title = asamashi_word.keyword + '・' + self.title
            self.link += str(asamashi_word.id) + '/'
            asamashi_rakuten_xml, created \
                = AsamashiRakutenXML.objects.get_or_create(genre_id=bits[0], asamashi_word=asamashi_word,
                                                           sort='-updateTimestamp', page_count=1,
                                                           min_price__isnull=True, max_price__isnull=True)
        self.link += 'utd/'
        self.pubdate = asamashi_rakuten_xml.modified
        return asamashi_rakuten_xml
