# -*- coding: utf-8 -*-
import urllib, time, random
from xml.etree import ElementTree

BASE_REQUEST_URL = 'http://api.rakuten.co.jp/rws/1.8/rest'
VERSION = '2007-04-11'

OPERATION_NAME_TABLE = {
    'ITEM_SEARCH': 'ItemSearch',
    'GENRE_SEARCH': 'GenreSearch'
}

COMMON_PARAM_TABLE = {
    'DEVELOPER_ID': 'developerId',
    'AFFILIATE_ID': 'affiliateId',
    'OPERATION': 'operation',
    'CALL_BACK': 'callBack'
}

COMMON_RESULT_TABLE = {
    'ARGS_LABEL': 'Args',
    'USER_AGENT': 'User-Agent',
    'DEVELOPER_ID': 'developerId',
    'OPERATION': 'operation',
    'VERSION': 'version',
    'STATUS': 'Status',
    'STATUS_MSG': 'StatusMsg'
}

ITEM_SEARCH_PARAM_TABLE = {
    'KEYWORD': 'keyword',
    'VERSION': 'version',
    'SHOP_CODE': 'shopCode',
    'GENRE_ID': 'genreId',
    'CATALOG_CODE': 'catalogCode',
    'HITS': 'hits',
    'PAGE': 'page',
    'SORT': 'sort',
    'MIN_PRICE': 'minPrice',
    'MAX_PRICE': 'maxPrice',
    'AVAILABILITY' : 'availability',
    'FIELD': 'field',
    'CARRIER': 'carrier',
    'IMAGE_FLAG': 'imageFlag',
    'OR_FLAG': 'orFlag',
    'NG_KEYWORD': 'NGKeyword'
}

GENRE_SEARCH_PARAM_TABLE = {
    'GENRE_ID': 'genreId',
    'VERSION': 'version',
    'GENRE_PATH': 'genrePath'
}

ITEM_SEARCH_SORT_PARAM_TABLE = {
    'AFFILIATE_RATE_ASC': '+affiliateRate',
    'AFFILIATE_RATE_DESC': '-affiliateRate',
    'REVIEW_COUNT_ASC': '+reviewCount',
    'REVIEW_COUNT_DESC': '-reviewCount',
    'ITEM_PRICE_ASC': '+itemPrice',
    'ITEM_PRICE_DESC': '-itemPrice',
    'UPDATE_TIMESTAMP_ASC': '+updateTimestamp',
    'UPDATE_TIMESTAMP_DESC': '-updateTimestamp',
    'RANDOM': 'random'
}

ITEM_SEARCH_RESULT_TABLE = {
    'COUNT': 'count',
    'PAGE': 'page',
    'FIRST': 'first',
    'LAST': 'last',
    'HITS': 'hits',
    'CARRIER': 'carrier',
    'PAGE_COUNT': 'pageCount',
    'ITEM_LABEL': 'Item',
    'ITEM_NAME': 'itemName',
    'ITEM_CODE': 'itemCode',
    'ITEM_PRICE': 'itemPrice',
    'ITEM_CAPTION': 'itemCaption',
    'ITEM_URL': 'itemUrl',
    'AFFILIATE_URL': 'affiliateUrl',
    'IMAGE_FLAG': 'imageFlag',
    'SMALL_IMAGE_URL': 'smallImageUrl',
    'MEDIUM_IMAGE_URL': 'mediumImageUrl',
    'AVAILABILITY': 'availability',
    'TAX_FLAG': 'taxFlag',
    'POSTAGE_FLAG': 'postageFlag',
    'CREDIT_CARD_FLAG': 'creditCardFlag',
    'SHOP_OF_THE_YEAR_FLAG': 'shopOfTheYearFlag',
    'AFFILIATE_RATE': 'affiliateRate',
    'START_TIME': 'startTime',
    'END_TIME': 'endTime',
    'REVIEW_COUNT': 'reviewCount',
    'REVIEW_AVERAGE': 'reviewAverage',
    'SHOP_NAME': 'shopName',
    'SHOP_CODE': 'shopCode',
    'SHOP_URL': 'shopUrl',
    'GENRE_ID': 'genreId'
}

GENRE_SEARCH_RESULT_TABLE = {
    'PARENT_LABEL': 'parent',
    'CURRENT_LABEL': 'current',
    'CHILD_LABEL': 'child',
    'GENRE_ID': 'genreId',
    'GENRE_NAME': 'genreName',
    'GENRE_LEVEL': 'genreLevel'
}

class RakutenError(StandardError): pass
class NotFoundError(RakutenError): pass
class ServerError(RakutenError): pass
class ClientError(RakutenError): pass
class MaintenanceError(RakutenError): pass

STATUS_MESSAGE_TABLE = {
    'Success': None,
    'NotFound': NotFoundError,
    'ServerError': ServerError,
    'ClientError': ClientError,
    'Maintenance': MaintenanceError
}


def request_url(query):
    return '%s?%s' % (BASE_REQUEST_URL, urllib.urlencode(query))

def fetch(request):
    url = request_url(request.query())
    time.sleep(3*random.uniform(0.34, 1))
    #print url
    xml = urllib.urlopen(url).read()
    count = 0
    while xml == '503 Service Unavailable\n' and count < 5:
        #print 'refetch'
        time.sleep(5*random.uniform(0.2, 1))
        xml = urllib.urlopen(url).read()
        count += 1
        #print count
    return xml


class Request(object):

    keyword = None
    version = VERSION
    shop_code = None
    genre_id = None
    catalog_code = None
    hits = None
    page = None
    sort = None
    min_price = None
    max_price = None
    availability = None
    field = None
    carrier = None
    image_flag = None
    or_flag = None
    ng_keyword = None

    genre_id = None
    genre_path = None

    def __init__(self, op, dev_id, aff_id=None, **opts):
        self.op = op
        self.dev_id = dev_id
        self.aff_id = aff_id
        self.set_attrs(opts)

    def set_attrs(self, adict):
        for k, v in adict.items():
            setattr(self, k, v)

    def query(self):
        query_dict = {
            COMMON_PARAM_TABLE['OPERATION']: self.op,
            COMMON_PARAM_TABLE['DEVELOPER_ID']: self.dev_id,
            COMMON_PARAM_TABLE['AFFILIATE_ID']: self.aff_id
        }
        
        if self.op == OPERATION_NAME_TABLE['ITEM_SEARCH']:
            #adict = {ITEM_SEARCH_PARAM_TABLE['KEYWORD']: self.keyword,}
            query_dict.update({ITEM_SEARCH_PARAM_TABLE['KEYWORD']: self.keyword,
                               ITEM_SEARCH_PARAM_TABLE['VERSION']: self.version,
                               ITEM_SEARCH_PARAM_TABLE['SHOP_CODE']: self.shop_code,
                               ITEM_SEARCH_PARAM_TABLE['GENRE_ID']: self.genre_id,
                               ITEM_SEARCH_PARAM_TABLE['CATALOG_CODE']: self.catalog_code,
                               ITEM_SEARCH_PARAM_TABLE['HITS']: self.hits,
                               ITEM_SEARCH_PARAM_TABLE['PAGE']: self.page,
                               #ITEM_SEARCH_PARAM_TABLE['SORT']: ITEM_SEARCH_SORT_PARAM_TABLE[self.sort],
                               ITEM_SEARCH_PARAM_TABLE['SORT']: self.sort,
                               ITEM_SEARCH_PARAM_TABLE['MIN_PRICE']: self.min_price,
                               ITEM_SEARCH_PARAM_TABLE['MAX_PRICE']: self.max_price,
                               ITEM_SEARCH_PARAM_TABLE['AVAILABILITY']: self.availability,
                               ITEM_SEARCH_PARAM_TABLE['FIELD']: self.field,
                               ITEM_SEARCH_PARAM_TABLE['CARRIER']: self.carrier,
                               ITEM_SEARCH_PARAM_TABLE['IMAGE_FLAG']: self.image_flag,
                               ITEM_SEARCH_PARAM_TABLE['OR_FLAG']: self.or_flag,
                               ITEM_SEARCH_PARAM_TABLE['NG_KEYWORD']: self.ng_keyword},)
        elif self.op == OPERATION_NAME_TABLE['GENRE_SEARCH']:
            query_dict.update({GENRE_SEARCH_PARAM_TABLE['GENRE_ID']: self.genre_id,
                               GENRE_SEARCH_PARAM_TABLE['VERSION']: self.version,
                               GENRE_SEARCH_PARAM_TABLE['GENRE_PATH']: self.genre_path},)
        
        query = {}
        for k, v in query_dict.items():
            if v is None:
                pass
            else:
                query[k] = v

        return query

    
class Response(object):

    status_error = None

    def __init__(self, xml=None):
        #self.url = url
        self.content = xml
        #self.items = []
        if xml: self.parse(xml)

    def parse(self, xml):
        try:
            et = ElementTree.fromstring(xml)
            args = et.find('.//' + COMMON_RESULT_TABLE['ARGS_LABEL'])
            for arg in args.getchildren():
                try:
                    key = arg.attrib['key']
                    value = arg.attrib['value']
                    if key == COMMON_RESULT_TABLE['USER_AGENT']:
                        self.user_agent = value
                    elif key == COMMON_RESULT_TABLE['DEVELOPER_ID']:
                        self.developer_id = value
                    elif key == COMMON_RESULT_TABLE['OPERATION']:
                        self.operation = value
                    elif key == COMMON_RESULT_TABLE['VERSION']:
                        self.version = value
                except Exception.KeyError:
                    pass
                
            self.status = et.find('.//' + COMMON_RESULT_TABLE['STATUS']).text
            if not self.status == 'Success': raise STATUS_MESSAGE_TABLE[self.status]
            self.status_message = et.find('.//' + COMMON_RESULT_TABLE['STATUS_MSG']).text

            if self.operation == OPERATION_NAME_TABLE['ITEM_SEARCH']:
                self.count = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['COUNT']).text
                self.page = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['PAGE']).text
                self.first = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['FIRST']).text
                self.last = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['LAST']).text
                self.hits = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['HITS']).text
                self.carrier = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['CARRIER']).text
                self.page_count = et.find('.//' + ITEM_SEARCH_RESULT_TABLE['PAGE_COUNT']).text

                self.items = []
                for el in et.findall('.//' + ITEM_SEARCH_RESULT_TABLE['ITEM_LABEL']):
                    item = {'item_name': el.find(ITEM_SEARCH_RESULT_TABLE['ITEM_NAME']).text,
                            'item_code': el.find(ITEM_SEARCH_RESULT_TABLE['ITEM_CODE']).text,
                            'item_price': el.find(ITEM_SEARCH_RESULT_TABLE['ITEM_PRICE']).text,
                            'item_caption': el.find(ITEM_SEARCH_RESULT_TABLE['ITEM_CAPTION']).text,
                            'item_url': el.find(ITEM_SEARCH_RESULT_TABLE['ITEM_URL']).text,
                            'affiliate_url': el.find(ITEM_SEARCH_RESULT_TABLE['AFFILIATE_URL']).text,
                            'image_flag': el.find(ITEM_SEARCH_RESULT_TABLE['IMAGE_FLAG']).text,
                            'small_image_url': el.find(ITEM_SEARCH_RESULT_TABLE['SMALL_IMAGE_URL']).text,
                            'medium_image_url': el.find(ITEM_SEARCH_RESULT_TABLE['MEDIUM_IMAGE_URL']).text,
                            'availability': el.find(ITEM_SEARCH_RESULT_TABLE['AVAILABILITY']).text,
                            'tax_flag': el.find(ITEM_SEARCH_RESULT_TABLE['TAX_FLAG']).text,
                            'postage_flag': el.find(ITEM_SEARCH_RESULT_TABLE['POSTAGE_FLAG']).text,
                            'credit_card_flag': el.find(ITEM_SEARCH_RESULT_TABLE['CREDIT_CARD_FLAG']).text,
                            'shop_of_the_year_flag': el.find(ITEM_SEARCH_RESULT_TABLE['SHOP_OF_THE_YEAR_FLAG']).text,
                            'affiliate_rate': el.find(ITEM_SEARCH_RESULT_TABLE['AFFILIATE_RATE']).text,
                            'start_time': el.find(ITEM_SEARCH_RESULT_TABLE['START_TIME']).text,
                            'end_time': el.find(ITEM_SEARCH_RESULT_TABLE['END_TIME']).text,
                            'review_count': el.find(ITEM_SEARCH_RESULT_TABLE['REVIEW_COUNT']).text,
                            'review_average': el.find(ITEM_SEARCH_RESULT_TABLE['REVIEW_AVERAGE']).text,
                            'shop_name': el.find(ITEM_SEARCH_RESULT_TABLE['SHOP_NAME']).text,
                            'shop_code': el.find(ITEM_SEARCH_RESULT_TABLE['SHOP_CODE']).text,
                            'shop_url': el.find(ITEM_SEARCH_RESULT_TABLE['SHOP_URL']).text,
                            'genre_id': el.find(ITEM_SEARCH_RESULT_TABLE['GENRE_ID']).text}
                    self.items.append(item)
            elif self.operation == OPERATION_NAME_TABLE['GENRE_SEARCH']:
                self.parents = []
                self.current_genre_id = None
                self.current_genre_name = None
                self.current_genre_level = None
                self.children = []
                
                if et.find('.//' + GENRE_SEARCH_RESULT_TABLE['PARENT_LABEL']):
                    for el in et.findall('.//' + GENRE_SEARCH_RESULT_TABLE['PARENT_LABEL']):
                        parent = {'genre_id': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_ID']).text,
                                  'genre_name': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_NAME']).text,
                                  'genre_level': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_LEVEL']).text}
                        self.parents.append(parent)

                if et.find('.//' + GENRE_SEARCH_RESULT_TABLE['CURRENT_LABEL']):
                    el = et.find('.//' + GENRE_SEARCH_RESULT_TABLE['CURRENT_LABEL'])
                    self.current_genre_id = el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_ID']).text
                    self.current_genre_name = el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_NAME']).text
                    self.current_genre_level = el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_LEVEL']).text

                if et.find('.//' + GENRE_SEARCH_RESULT_TABLE['CHILD_LABEL']):
                    for el in et.findall('.//' + GENRE_SEARCH_RESULT_TABLE['CHILD_LABEL']):
                        child = {'genre_id': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_ID']).text,
                                'genre_name': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_NAME']).text,
                                'genre_level': el.find('.//' + GENRE_SEARCH_RESULT_TABLE['GENRE_LEVEL']).text}
                        self.children.append(child)
        except:
            #traceback.print_exc()
            raise NotFoundError
