# -*- coding: utf-8 -*-
import urllib, datetime
from xml.etree import ElementTree

from django import template
from django.template import defaultfilters

from reiare.asamashie.models import AsamashiWord, AsamashiRakutenXML, AsamashiRakutenGenreXML
from reiare.asamashie.rakuten_access import ITEM_SEARCH_SORT_PARAM_TABLE


URL_PARAM_TO_ITEM_SEARCH_SORT_PARAM_TABLE = {
    'ara': 'AFFILIATE_RATE_ASC',
    'ard': 'AFFILIATE_RATE_DESC',
    'rca': 'REVIEW_COUNT_ASC',
    'rcd': 'REVIEW_COUNT_DESC',
    'ipa': 'ITEM_PRICE_ASC',
    'ipd': 'ITEM_PRICE_DESC',
    'uta': 'UPDATE_TIMESTAMP_ASC',
    'utd': 'UPDATE_TIMESTAMP_DESC',
    'rdm': 'RANDOM'
}

SORT_ORDER_DESCRIPTION_TABLE = {
    'ara': 'アフィリエイト料率が低い',
    'ard': 'アフィリエイト料率が高い',
    'rca': 'レビュー件数が少ない',
    'rcd': 'レビュー件数が多い',
    'ipa': '価格が安い',
    'ipd': '価格が高い',
    'uta': '更新日時が古い',
    'utd': '更新日時が新しい',
    'rdm': 'ランダム'
}

PRICE_ZONE_PARAM_TABLE = {
    'min1': '0',
    'max1': '999',
    'min2': '1000',
    'max2': '2999',
    'min3': '3000',
    'max3': '4999',
    'min4': '5000',
    'max4': '9999',
    'min5': '10000',
    'max5': '29999',
    'min6': '30000',
    'max6': '49999',
    'min7': '50000',
    'max7': '99999',
    'min8': '100000',
    'max8': '499999',
    'min9': '500000',
    'max9': '999999'
}

register = template.Library()

@register.inclusion_tag('rakuten_item_search_result.html')
def show_rakuten_item_search_result(asamashi_word=None, page=1, sort='ipa', price_zone=None, rakuten_genre=None):
    query_dict = {'page_count': page}
    if asamashi_word:
        query_dict['asamashi_word'] = asamashi_word
    else:
        query_dict['asamashi_word__isnull'] = True

    sort_order = ITEM_SEARCH_SORT_PARAM_TABLE[URL_PARAM_TO_ITEM_SEARCH_SORT_PARAM_TABLE[sort]]
    if not sort_order:
        sort_order = ITEM_SEARCH_SORT_RARAM_TABLE['ITEM_PRICE_ASC']
    query_dict['sort'] = sort_order

    if price_zone:
        min_price = None
        max_price = None

        min_price = PRICE_ZONE_PARAM_TABLE['min' + price_zone]
        max_price = PRICE_ZONE_PARAM_TABLE['max' + price_zone]

        if min_price and max_price:
            query_dict['min_price'] = min_price
            query_dict['max_price'] = max_price
        else:
            query_dict['min_price__isnull'] = True
            query_dict['max_price__isnull'] = True
    else:
        query_dict['min_price__isnull'] = True
        query_dict['max_price__isnull'] = True

    if rakuten_genre and rakuten_genre['current_genre_id']:
        query_dict['genre_id'] = rakuten_genre['current_genre_id']
    else:
        query_dict['genre_id__isnull'] = True

    try:
        asamashi_rakuten_xml, created = AsamashiRakutenXML.objects.get_or_create(**query_dict)
    except AssertionError:
        asamashi_rakuten_xml = AsamashiRakutenXML.objects.filter(**query_dict)[1]
    except AsamashiRakutenXML.DoesNotExist:
        return HttpResponseRedirect('/asamashie/')
        
    dict = asamashi_rakuten_xml.get_dict()
    if dict:
        dict['modified'] = asamashi_rakuten_xml.modified
        #dict.update({'modified': asamashi_rakuten_xml.modified})

    return {'results': dict,
            'object': asamashi_word,
            'sort': sort,
            'price_zone': price_zone,
            'genre': rakuten_genre}

@register.inclusion_tag('asamashie_paginator.html', takes_context=True)
def asamashie_paginator(context, adjacent_pages=10):
    dict = context['results']
    if dict:
        dict.update({
                'page_counts': [n for n in \
                    range(int(dict['page']) - adjacent_pages, int(dict['page']) + adjacent_pages + 1) \
                    if n > 0 and n <= int(dict['page_count'])],
                'page_count': int(dict['page_count'])
            })
    return {
        'results': context['results'],
        'object': context['object'],
        'sort': context['sort'],
        'price_zone': context['price_zone'],
        'genre': context['genre']
    }

@register.filter(name='sort_description')
def sort_description(sort):
    return SORT_ORDER_DESCRIPTION_TABLE[sort]

@register.filter(name='price_zone_description')
def price_zone_description(price_zone):
    if price_zone:
        return PRICE_ZONE_PARAM_TABLE['min' + price_zone] + ' 〜 ' + \
            PRICE_ZONE_PARAM_TABLE['max' + price_zone] + '円'
    else:
        return 'すべて'

