# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *
from django.conf import settings

from blog.models import Entry, EntryArchive, EntryTag
from blog.feeds import LatestEntries, LatestEntriesByTag
from blog.forms import SearchForm

info_dict = {
    'queryset': Entry.published_objects.all().select_related(),
    'date_field': 'created'
}

feeds = {
    'latest': LatestEntries,
    'tags': LatestEntriesByTag,
    'tag': LatestEntriesByTag
}

urlpatterns = patterns('reiare.blog.views',
    (r'^tag/(?P<tag>.*)/image/$', 'tag_image_list'),
    (r'^tag/(?P<tag>.*)/$', 'tag_list'),
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>.*)/post_comment/$', 'post_comment'),
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>.*)/$', 'detail'),
    (r'^comment/(?P<object_id>\d+)/$', 'get_comment'),
    (r'^post_comment/(?P<object_id>\d+)/$', 'post_comment_ajax'),
    (r'^asin2asamashi/(?P<associate_tag>.*)/(?P<asins>.*)/$', 'asin2asamashi'),
    (r'^body/(?P<object_id>\d+)/$', 'get_body'),
    (r'^entry/preview/$', 'get_blog_preview'),
    (r'^search/$', 'get_search_result'),
    (r'^lastfm/recent_tracks/$', 'get_lastfm_recent_tracks'),
    (r'^lastfm/recent_weekly_track_chart/$', 'get_lastfm_recent_weekly_track_chart'),
    (r'^lastfm/recent_weekly_artist_chart/$', 'get_lastfm_recent_weekly_artist_chart'),
    (r'^lastfm/top_artists/$', 'get_lastfm_top_artists'),
    (r'^lastfm/top_tracks/$', 'get_lastfm_top_tracks'),
    (r'^recent_entries/(?P<page_num>\d+)/$', 'get_recent_entries'),
    (r'^recent_comments/(?P<page_num>\d+)/$', 'get_recent_comments'),
)

urlpatterns += patterns('django.views.generic.date_based',
    (r'^$', 'archive_index', dict(info_dict, num_latest=5,
                                  extra_context = {
                                      'search_form': SearchForm(),
                                  })),
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/$', 'archive_month', dict(info_dict, month_format='%m', allow_empty=True,)),
)

urlpatterns += patterns('',
    (r'^feeds/(?P<url>.*)/$', 'django.contrib.syndication.views.feed', {'feed_dict': feeds}),
)
